@@ -6,11 +6,12 @@ use warnings;
 use vars qw/$AUTOLOAD/;
 
 use Carp;
-use Storable qw/freeze/;
 use Module::Load;
 
 use parent 'ActiveRecord::Simple';
 
+use ActiveRecord::Simple::Utils;
+
 
 our $MAXIMUM_LIMIT = 100_000_000_000;
 
@@ -24,6 +25,9 @@ sub new {
     my $table_name = ($self->{class}->can('_get_table_name'))  ? $self->{class}->_get_table_name  : undef;
     my $pkey       = ($self->{class}->can('_get_primary_key')) ? $self->{class}->_get_primary_key : undef;
 
+    croak 'can not get table_name for class ' . $self->{class} unless $table_name;
+    #croak 'can not get primary_key for class ' . $self->{class} unless $pkey;
+
     $self->{prep_select_fields} //= [];
     $self->{prep_select_from}   //= [];
     $self->{prep_select_where}  //= [];
@@ -45,7 +49,7 @@ sub new {
     }
     elsif (ref $param[0] && ref $param[0] eq 'HASH') {
         # find many by params
-        my ($bind, $condition_pairs) = $self->parse_hash($param[0]);
+        my ($bind, $condition_pairs) = $self->_parse_hash($param[0]);
 
         my $where_str = join q/ AND /, @$condition_pairs;
 
@@ -94,7 +98,7 @@ sub count {
         s/"//g foreach @group_by;
         my @results;
         foreach my $item ($self->fetch) {
-            push my @line, (count => $item->{'COUNT(*)'}), map {$_ => $item->$_} @group_by;
+            push my @line, (count => $item->{'COUNT(*)'}), map { $_ => $item->{$_} } @group_by;
             push @results, { @line };
         }
         return @results;
@@ -104,64 +108,6 @@ sub count {
     }
 }
 
-sub parse_hash {
-    my ($self, $param_hash) = @_;
-    my $class = $self->{class};
-    my $table_name = ($self->{class}->can('_get_table_name'))  ? $self->{class}->_get_table_name  : undef;
-    my ($bind, $condition_pairs) = ([],[]);
-    for my $param_name (keys %{ $param_hash }) {
-        if (ref $param_hash->{$param_name} eq 'ARRAY' and !ref $param_hash->{$param_name}[0]) {
-            my $instr = join q/, /, map { '?' } @{ $param_hash->{$param_name} };
-            push @$condition_pairs, qq/"$table_name"."$param_name" IN ($instr)/;
-            push @$bind, @{ $param_hash->{$param_name} };
-        }
-        elsif (ref $param_hash->{$param_name}) {
-            next if !$class->can('_get_relations');
-            my $relation = $class->_get_relations->{$param_name} or next;
-
-            next if $relation->{type} ne 'one';
-            my $fk = $relation->{params}{fk};
-            my $pk = $relation->{params}{pk};
-
-            if (ref $param_hash->{$param_name} eq __PACKAGE__) {
-                my $object = $param_hash->{$param_name};
-
-                my $tmp_table = qq/tmp_table_/ . sprintf("%x", $object);
-                my $request_table = $object->{class}->_get_table_name;
-
-                $object->{prep_select_fields} = [qq/"$request_table"."$pk"/];
-                $object->_finish_sql_stmt;
-
-                push @$condition_pairs, qq/"$table_name"."$fk" IN (SELECT "$tmp_table"."$pk" from ($object->{SQL}) as $tmp_table)/;
-                push @$bind, @{ $object->{BIND} } if ref $object->{BIND} eq 'ARRAY';
-            }
-            else {
-                my $object = $param_hash->{$param_name};
-
-                if (ref $object eq 'ARRAY') {
-                    push @$bind, map $_->$pk, @$object;
-                    push @$condition_pairs, qq/"$table_name"."$fk" IN (@{[ join ', ', map "?", @$object ]})/;
-                }
-                else {
-                    push @$condition_pairs, qq/"$table_name"."$fk" = ?/;
-                    push @$bind, $object->$pk;
-                }
-            }
-        }
-        else {
-            if (defined $param_hash->{$param_name}) {
-                push @$condition_pairs, qq/"$table_name"."$param_name" = ?/;
-                push @$bind, $param_hash->{$param_name};
-            }
-            else {
-                # is NULL
-                push @$condition_pairs, qq/"$table_name"."$param_name" IS NULL/;
-            }
-        }
-    }
-    return ($bind, $condition_pairs);
-}
-
 sub first {
     my ($self, $limit) = @_;
 
@@ -201,7 +147,7 @@ sub only {
         grep { $_ ne qq/"$table_name".*/ } @{ $self->{prep_select_fields} };
     for my $fld (@fields) {
         if ($mixins && grep { $_ eq $fld } keys %$mixins) {
-            my $mixin = $mixins->{$fld}->();
+            my $mixin = $mixins->{$fld}->($self->{class});
             $mixin .= qq/ AS $fld/ unless $mixin =~ /as\s+\w+$/i;
             push @filtered_prep_select_fields, $mixin;
         }
@@ -230,30 +176,11 @@ sub order_by {
 }
 
 sub desc {
-    return shift->order_by_direction('DESC');
+    return shift->_order_by_direction('DESC');
 }
 
 sub asc {
-    return shift->order_by_direction('ASC');
-}
-
-sub order_by_direction {
-    my ($self, $direction) = @_;
-
-    # There are no fields for order yet
-    return unless ref $self->{prep_order_by} eq 'ARRAY' and scalar @{ $self->{prep_order_by} } > 0;
-
-    # asc/desc is called before: ->asc->desc
-    return if defined $self->{prep_asc_desc};
-
-    # $direction should be ASC/DESC
-    return unless $direction =~ /^(ASC|DESC)$/i;
-
-    # Add $direction to the latest field
-    @{$self->{prep_order_by}}[-1] .= " $direction";
-    $self->{prep_asc_desc} = 1;
-
-    return $self;
+    return shift->_order_by_direction('ASC');
 }
 
 sub group_by {
@@ -286,121 +213,6 @@ sub offset {
     return $self;
 }
 
-sub abstract {
-    my ($self, $opts) = @_;
-
-    return $self if ! ref $opts && ref $opts ne 'HASH';
-
-    while (my ($method, $param) = each %$opts) {
-        if ($method eq 'order_by') {
-            $self->order_by(@{ $param->{columns} });
-            my $order_direction = (defined $param->{direction}) ? $param->{direction} : undef;
-            $self->$order_direction if $order_direction;
-        }
-        else {
-            my @p = (ref $param) ? @$param : ($param);
-            $self->$method(@p);
-        }
-    }
-
-    return $self;
-}
-
-sub select {
-    my ($self_class, $class, @params) = @_;
-
-    my @find_params;
-    my $abstract_params_hashref;
-
-    my $first_param = shift @params;
-    push @find_params, $first_param if defined $first_param;
-
-    for my $param (@params) {
-        #push @find_params, $param if ref $param ne 'HASH';
-        if (ref $param eq 'HASH') {
-            $abstract_params_hashref = $param;
-            last;
-        }
-        else {
-            push @find_params, $param;
-        }
-    }
-
-    my $finder = $self_class->new($class, @find_params);
-    $finder->abstract($abstract_params_hashref);
-
-    return $finder->fetch;
-}
-
-
-sub _finish_sql_stmt {
-    my ($self) = @_;
-
-    ref $self->{prep_select_fields} or croak 'Invalid prepare SQL statement';
-    ref $self->{prep_select_from}   or croak 'Invalid prepare SQL statement';
-
-    my $table_name = $self->{class}->_get_table_name;
-    my @add = grep { $_ !~~ $self->{prep_select_fields} } map qq/"$table_name".$_/, @{ $self->{prep_group_by}||[] };
-    push @{ $self->{prep_select_fields} }, @add;
-
-    $self->{SQL} = "SELECT " . (join q/, /, @{ $self->{prep_select_fields} }) . "\n";
-    $self->{SQL} .= "FROM " . (join q/, /, @{ $self->{prep_select_from} }) . "\n";
-
-    if (defined $self->{prep_left_joins}) {
-        $self->{SQL} .= "$_\n" for @{ $self->{prep_left_joins} };
-        $self->{has_joined_table} = 1;
-    }
-
-    if (@{ $self->{prep_select_where}||[] }) {
-        $self->{SQL} .= "WHERE\n";
-        $self->{SQL} .= join " AND ", @{ $self->{prep_select_where} };
-    }
-
-    if (@{ $self->{prep_group_by}||[] }) {
-        $self->{SQL} .= ' GROUP BY ';
-        $self->{SQL} .= join q/, /, @{ $self->{prep_group_by} };
-    }
-
-    if (@{ $self->{prep_order_by}||[] }) {
-        $self->{SQL} .= ' ORDER BY ';
-        $self->{SQL} .= join q/, /, @{ $self->{prep_order_by} };
-    }
-
-    $self->{SQL} .= ' LIMIT ' .  ($self->{prep_limit}  // $MAXIMUM_LIMIT);
-    $self->{SQL} .= ' OFFSET '.  ($self->{prep_offset} // 0);
-
-    return $self;
-}
-
-sub _finish_object_representation {
-    my ($self, $obj, $object_data, $read_only) = @_;
-
-    if ($self->{has_joined_table}) {
-        RELATION:
-        for my $rel_name (@{ $self->{with} }) {
-            my $relation = $self->{class}->_get_relations->{$rel_name} or next RELATION;
-            my %pairs = map { $_, $object_data->{$_} } grep { $_ =~ /^JOINED\_$rel_name\_/ } keys %$object_data;
-            next RELATION unless %pairs;
-
-            for my $key (keys %pairs) {
-                my $val = delete $pairs{$key};
-                $key =~ s/^JOINED\_$rel_name\_//;
-                $pairs{$key} = $val;
-            }
-            $obj->{"relation_instance_$rel_name"} = $relation->{class}->new(\%pairs);
-
-            $obj->_delete_keys(qr/^JOINED\_$rel_name/);
-        }
-
-    }
-
-    $obj->{read_only} = 1 if defined $read_only;
-    $obj->{snapshoot} = freeze($object_data) if $obj->can('_smart_saving_used') && $obj->_smart_saving_used;
-    $obj->{isin_database} = 1;
-
-    return $obj;
-}
-
 sub fetch {
     my ($self, $param) = @_;
 
@@ -547,30 +359,54 @@ sub _find_many_to_many {
     my $class_opts = {};
     my $root_class_opts = {};
 
-    eval { load $param->{m_class} };
+    if ($param->{m_class}) {
+        eval { load $param->{m_class} };
 
-    for my $opts ( values %{ $param->{m_class}->_get_relations } ) {
-        if ($opts->{class} eq $param->{root_class}) {
-            $root_class_opts = $opts;
-        }
-        elsif ($opts->{class} eq $class) {
-            $class_opts = $opts;
+
+
+        for my $opts ( values %{ $param->{m_class}->_get_relations } ) {
+            if ($opts->{class} eq $param->{root_class}) {
+                $root_class_opts = $opts;
+            }
+            elsif ($opts->{class} eq $class) {
+                $class_opts = $opts;
+            }
         }
+
+        my $self = $self_class->new($class, @{ $param->{where_statement} });
+
+        my $connected_table_name = $class->_get_table_name;
+        $self->{prep_select_from} = [ $param->{m_class}->_get_table_name ];
+
+        push @{ $self->{prep_left_joins} },
+            'JOIN ' . $connected_table_name . ' ON ' . $connected_table_name . '.' . $class->_get_primary_key . ' = '
+                . $param->{m_class}->_get_table_name . '.' . $class_opts->{params}{fk};
+
+        push @{ $self->{prep_select_where} },
+            $root_class_opts->{params}{fk} . ' = ' . $param->{self}->{ $param->{root_class}->_get_primary_key };
+
+        return $self;
+    }
+    else {
+        my $self = $self_class->new($class, @{ $param->{where_statement} });
+
+        my $connected_table_name = $class->_get_table_name;
+        $self->{prep_select_from} = [ $param->{via_table} ];
+        my $fk = ActiveRecord::Simple::Utils::class_to_table_name($class);
+        $fk .= '_id';
+
+        push @{ $self->{prep_left_joins} },
+            'JOIN ' . $connected_table_name . ' ON ' . $connected_table_name . '.' . $class->_get_primary_key . ' = '
+                . $param->{via_table} . '.' . $fk;
+
+        my $fk2 = ActiveRecord::Simple::Utils::class_to_table_name($param->{root_class}) . '_id';
+
+        push @{ $self->{prep_select_where} },
+            $fk2 . ' = ' . $param->{self}->{ $param->{root_class}->_get_primary_key };
+
+        return $self;
     }
 
-    my $self = $self_class->new($class, @{ $param->{where_statement} });
-
-    my $connected_table_name = $class->_get_table_name;
-    $self->{prep_select_from} = [ $param->{m_class}->_get_table_name ];
-
-    push @{ $self->{prep_left_joins} },
-        'JOIN ' . $connected_table_name . ' ON ' . $connected_table_name . '.' . $class->_get_primary_key . ' = '
-            . $param->{m_class}->_get_table_name . '.' . $class_opts->{params}{fk};
-
-    push @{ $self->{prep_select_where} },
-        $root_class_opts->{params}{fk} . ' = ' . $param->{self}->{ $param->{root_class}->_get_primary_key };
-
-    return $self;
 }
 
 sub _get_slice {
@@ -608,6 +444,150 @@ sub _quote_sql_stmt {
     return $self;
 }
 
+sub _finish_object_representation {
+    my ($self, $obj, $object_data, $read_only) = @_;
+
+    if ($self->{has_joined_table}) {
+        RELATION:
+        for my $rel_name (@{ $self->{with} }) {
+            my $relation = $self->{class}->_get_relations->{$rel_name} or next RELATION;
+            my %pairs = map { $_, $object_data->{$_} } grep { $_ =~ /^JOINED\_$rel_name\_/ } keys %$object_data;
+            next RELATION unless %pairs;
+
+            for my $key (keys %pairs) {
+                my $val = delete $pairs{$key};
+                $key =~ s/^JOINED\_$rel_name\_//;
+                $pairs{$key} = $val;
+            }
+            $obj->{"relation_instance_$rel_name"} = $relation->{class}->new(\%pairs);
+
+            $obj->_delete_keys(qr/^JOINED\_$rel_name/);
+        }
+
+    }
+
+    $obj->{read_only} = 1 if defined $read_only;
+    $obj->{isin_database} = 1;
+
+    return $obj;
+}
+
+sub _finish_sql_stmt {
+    my ($self) = @_;
+
+    ref $self->{prep_select_fields} or croak 'Invalid prepare SQL statement';
+    ref $self->{prep_select_from}   or croak 'Invalid prepare SQL statement';
+
+    my $table_name = $self->{class}->_get_table_name;
+    my @add = grep { $_ !~~ $self->{prep_select_fields} } map qq/"$table_name".$_/, @{ $self->{prep_group_by}||[] };
+    push @{ $self->{prep_select_fields} }, @add;
+
+    $self->{SQL} = "SELECT " . (join q/, /, @{ $self->{prep_select_fields} }) . "\n";
+    $self->{SQL} .= "FROM " . (join q/, /, @{ $self->{prep_select_from} }) . "\n";
+
+    if (defined $self->{prep_left_joins}) {
+        $self->{SQL} .= "$_\n" for @{ $self->{prep_left_joins} };
+        $self->{has_joined_table} = 1;
+    }
+
+    if (@{ $self->{prep_select_where}||[] }) {
+        $self->{SQL} .= "WHERE\n";
+        $self->{SQL} .= join " AND ", @{ $self->{prep_select_where} };
+    }
+
+    if (@{ $self->{prep_group_by}||[] }) {
+        $self->{SQL} .= ' GROUP BY ';
+        $self->{SQL} .= join q/, /, @{ $self->{prep_group_by} };
+    }
+
+    if (@{ $self->{prep_order_by}||[] }) {
+        $self->{SQL} .= ' ORDER BY ';
+        $self->{SQL} .= join q/, /, @{ $self->{prep_order_by} };
+    }
+
+    $self->{SQL} .= ' LIMIT ' .  ($self->{prep_limit}  // $MAXIMUM_LIMIT);
+    $self->{SQL} .= ' OFFSET '.  ($self->{prep_offset} // 0);
+
+    return $self;
+}
+
+sub _parse_hash {
+    my ($self, $param_hash) = @_;
+    my $class = $self->{class};
+    my $table_name = ($self->{class}->can('_get_table_name'))  ? $self->{class}->_get_table_name  : undef;
+    my ($bind, $condition_pairs) = ([],[]);
+    for my $param_name (keys %{ $param_hash }) {
+        if (ref $param_hash->{$param_name} eq 'ARRAY' and !ref $param_hash->{$param_name}[0]) {
+            my $instr = join q/, /, map { '?' } @{ $param_hash->{$param_name} };
+            push @$condition_pairs, qq/"$table_name"."$param_name" IN ($instr)/;
+            push @$bind, @{ $param_hash->{$param_name} };
+        }
+        elsif (ref $param_hash->{$param_name}) {
+            next if !$class->can('_get_relations');
+            my $relation = $class->_get_relations->{$param_name} or next;
+
+            next if $relation->{type} ne 'one';
+            my $fk = $relation->{params}{fk};
+            my $pk = $relation->{params}{pk};
+
+            if (ref $param_hash->{$param_name} eq __PACKAGE__) {
+                my $object = $param_hash->{$param_name};
+
+                my $tmp_table = qq/tmp_table_/ . sprintf("%x", $object);
+                my $request_table = $object->{class}->_get_table_name;
+
+                $object->{prep_select_fields} = [qq/"$request_table"."$pk"/];
+                $object->_finish_sql_stmt;
+
+                push @$condition_pairs, qq/"$table_name"."$fk" IN (SELECT "$tmp_table"."$pk" from ($object->{SQL}) as $tmp_table)/;
+                push @$bind, @{ $object->{BIND} } if ref $object->{BIND} eq 'ARRAY';
+            }
+            else {
+                my $object = $param_hash->{$param_name};
+
+                if (ref $object eq 'ARRAY') {
+                    push @$bind, map $_->$pk, @$object;
+                    push @$condition_pairs, qq/"$table_name"."$fk" IN (@{[ join ', ', map "?", @$object ]})/;
+                }
+                else {
+                    push @$condition_pairs, qq/"$table_name"."$fk" = ?/;
+                    push @$bind, $object->$pk;
+                }
+            }
+        }
+        else {
+            if (defined $param_hash->{$param_name}) {
+                push @$condition_pairs, qq/"$table_name"."$param_name" = ?/;
+                push @$bind, $param_hash->{$param_name};
+            }
+            else {
+                # is NULL
+                push @$condition_pairs, qq/"$table_name"."$param_name" IS NULL/;
+            }
+        }
+    }
+    return ($bind, $condition_pairs);
+}
+
+sub _order_by_direction {
+    my ($self, $direction) = @_;
+
+    # There are no fields for order yet
+    return unless ref $self->{prep_order_by} eq 'ARRAY' and scalar @{ $self->{prep_order_by} } > 0;
+
+    # asc/desc is called before: ->asc->desc
+    return if defined $self->{prep_asc_desc};
+
+    # $direction should be ASC/DESC
+    return unless $direction =~ /^(ASC|DESC)$/i;
+
+    # Add $direction to the latest field
+    @{$self->{prep_order_by}}[-1] .= " $direction";
+    $self->{prep_asc_desc} = 1;
+
+    return $self;
+}
+
 sub DESTROY { }
 
 sub AUTOLOAD {
@@ -623,4 +603,238 @@ sub AUTOLOAD {
 
 1;
 
+__END__;
 
+
+=head1 NAME
+
+ActiveRecord::Simple::Find
+
+=head1 DESCRIPTION
+
+ActiveRecord::Simple is a simple lightweight implementation of ActiveRecord
+pattern. It's fast, very simple and very light.
+
+ActiveRecord::Simple::Find is a class to search, ordering, organize and fetch data from database.
+It generates SQL-code and iteracts with DBI to execute it.
+
+=head1 SYNOPSIS
+
+my @customers = Customer->find({ name => 'Bill' })->fetch;
+my @customers = Customer->find({ zip => [1001, 1002, 1003] })->fetch;
+my @customers = Customer->find('age > ?', 21)->fetch;
+my @customers = Customer->find([1, 2, 3, 4, 5])->order_by('id')->desc->fetch;
+
+
+=head1 METHODS
+
+L<ActiveRecord::Simple::Find> implements the following methods.
+
+=head2 new
+
+Object constructor, creates basic search pattern. Available from method "find"
+of the base class:
+
+    # SELECT * FROM log WHERE site_id = 1 AND level = 'error';
+    my $f = Log->find({ id => 1, level => 'error' });
+
+    # SELECT * FROM log WHERE site_id = 1 AND level IN ('error', 'warning');
+    my $f = Log->find({ id => 1, level => ['error', 'warning'] });
+
+    # SELECT * FROM customer WHERE age > 21;
+    Customer->find('age > ?', 21);
+
+    # SELECT * FROM customer WHERE id = 100;
+    Customer->find(100);
+
+    # SELECT * FROM customer WHERE id IN (100, 101, 191);
+    Customer->find([100, 101, 191]);
+
+=head2 last
+
+Fetch last row from database:
+
+    # get very last log:
+    my $last_log = Log->find->last; 
+
+    # get last error log of site number 1:
+    my $last_log = Log->find({ level => 'error', site_id => 1 })->last;
+
+=head2 first
+
+Fetch first row:
+
+    # get very first log:
+    my $first_log = Log->find->first; 
+
+    # get first error log of site number 1:
+    my $first_log = Log->find({ level => 'error', site_id => 1 })->first;
+
+=head2 count
+
+Fetch number of records in the database:
+
+    my $cnt = Log->find->count();
+    my $cnt_warnings = Log->find({ level => 'warnings' })->count;
+
+=head2 exists 
+
+Check the record is exist:
+
+    if (Log->find({ level => 'fatal' })->exists) {
+        die "got fatal error log!";
+    }
+
+=head2 fetch
+
+Fetch data from the database as objects:
+
+    my @errors = Log->find({ level => 'error' })->fetch;
+    my $errors = Log->find({ level => 'error' })->fetch; # the same, but returns ARRAY ref
+    my $error = Log->find(1)->fetch; # only one record
+    my @only_five_errors = Log->find({ level => 'error' })->fetch(5);
+
+=head2 next
+
+Fetch next n rows from the database:
+
+    my $finder = Log->find({ level => 'info' });
+    
+    # get logs by lists of 10 elements:
+    while (my @logs = $finder->next(10)) {
+        print $_->id, "\n" for @logs;
+    }
+
+=head2 only 
+
+Specify field names to get from database:
+
+    # SELECT id, message FROM log;
+    my @logs = Log->find->only('id', 'message');
+
+=head2 fields
+
+The same as "only":
+
+    # SELECT id, message FROM log;
+    my @logs = Log->find->fields('id', 'message');
+
+=head2 order_by
+
+Set "ORDER BY" command to the query:
+
+    # SELECT * FROM log ORDER BY inserted_time;
+    my @logs = Log->find->order_by('inserted_time');
+
+    # SELECT * FROM log ORDER BY level, id;
+    my @logs = Log->find->order_by('level', 'id');
+
+=head2 asc
+
+Set "ASC" to the query:
+
+    # SELECT * FROM log ORDER BY id ASC;
+    my @logs = Log->find->order_by('id')->asc;
+
+=head2 desc
+
+Set "DESC" to the query:
+
+    # SELECT * FROM log ORDER BY id DESC;
+    my @logs = Log->find->order_by('id')->desc;
+
+=head2 limit
+
+SET "LIMIT" to the query:
+
+    # SELECT * FROM log LIMIT 100;
+    my @logs = Log->find->limit(100);
+
+=head2 offset
+
+SET "OFFSET" to the query:
+
+    # SELECT * FROM log LIMIT 100 OFFSET 99;
+    my @logs = Log->find->limit(100)->offset(99);
+
+=head2 group_by
+
+Set "GROUP BY":
+
+    my @logs = Log->find->group_by('level');
+
+=head2 with 
+
+Set "LEFT JOIN" command to the query:
+
+    # SELECT l.*, s.* FROM logs l LEFT JOIN sites s ON s.id = l.site_id
+    my @logs_and_sites = Log->find->with('sites');
+    print $_->site->name, ": ", $_->mesage for @logs_and_sites;
+
+=head2 left_join
+
+The same as "with" method
+
+=head2 uplod
+
+Fetch object from database and load into ActiveRecord::Simple::Find object:
+
+    my $logs = Log->find({ level => ['error', 'fatal'] });
+    $logs->order_by('level')->desc;
+    $logs->limit(100);
+    $logs->upload;
+
+    print $_->message for @$logs;
+
+=head2 to_sql
+
+Show SQL-query that genereted by ActiveRecord::Simple::Find class:
+
+    my $finder = Log->frind->only('message')->order_by('level')->desc->limit(100);
+    print $finder->to_sql; # prints: SELECT message FROM log ORDER BY level DESC LIMIT 100;
+
+
+=head1 EXAMPLES
+
+    
+
+=head1 AUTHOR
+
+shootnix, C<< <shootnix at cpan.org> >>
+
+=head1 BUGS
+
+Please report any bugs or feature requests to C<shootnix@cpan.org>, or through
+the github: https://github.com/shootnix/activerecord-simple/issues
+
+=head1 SUPPORT
+
+You can find documentation for this module with the perldoc command.
+
+    perldoc ActiveRecord::Simple
+
+
+You can also look for information at:
+
+=over 1
+
+=item * Github wiki:
+
+L<https://github.com/shootnix/activerecord-simple/wiki>
+
+=back
+
+=head1 ACKNOWLEDGEMENTS
+
+
+=head1 LICENSE AND COPYRIGHT
+
+Copyright 2013-2018 shootnix.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of either: the GNU General Public License as published
+by the Free Software Foundation; or the Artistic License.
+
+See http://dev.perl.org/licenses/ for more information.
+
+=cut
